--[[

    File: tex.lua 

    Copyright (C) 2000-2009 Christopher Moore (christopher.e.moore@gmail.com)
	  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

--]]

require 'common.util'

local texWrapForKey = {
	s = gl.GL_TEXTURE_WRAP_S;
	t = gl.GL_TEXTURE_WRAP_T;
	r = gl.GL_TEXTURE_WRAP_R;
}

-- TODO - consider Tex:new rather than Tex.new
-- then all 'new' methods could reuse the same code ... even beyond just textures

Tex = class()
function Tex:init(args)
	self.id = gl.glGenTexture()
	if args.unit then
		gl.glActiveTexture(gl.GL_TEXTURE0 + unit)
	end
	self:bind()
	self:setArgs(args)
	self:create(args)	--abstract
end
function Tex:setArgs(args)
	if args.minFilter then
		gl.glTexParameter(self.target, gl.GL_TEXTURE_MIN_FILTER, args.minFilter)
	end
	if args.magFilter then
		gl.glTexParameter(self.target, gl.GL_TEXTURE_MAG_FILTER, args.magFilter)
	end
	if args.wrap then
		for k,v in pairs(args.wrap) do
			gl.glTexParameter(self.target, texWrapForKey[k] or k, v)
		end
	end
	if args.generateMipmap then
		gl.glTexParameter(self.target, gl.GL_GENERATE_MIPMAP, args.generateMipmap)
	end
end
function Tex:enable()
	gl.glEnable(self.target)
end
function Tex:disable()
	gl.glDisable(self.target)
end
function Tex:bind(unit)
	if unit then
		gl.glActiveTexture(gl.GL_TEXTURE0 + unit)
	end
	gl.glBindTexture(self.target, self.id)
end
function Tex:unbind(unit)
	if unit then
		gl.glActiveTexture(gl.GL_TEXTURE0 + unit)
	end
	gl.glBindTexture(self.target, 0)
end
function Tex:delete()
	gl.glDeleteTexture(self.id)
end


Tex1D = class(Tex)
Tex1D.target = gl.GL_TEXTURE_1D
function Tex1D:create(args)
	gl.glTexImage1D(
		self.target,
		args.level or 0,
		args.internalFormat or gl.GL_RGBA,
		args.width,
		args.border or 0,
		args.format or gl.GL_RGBA,
		args.type or gl.GL_FLOAT,
		args.data)
	-- store for later use
	self.width = args.width
end


Tex2D = class(Tex)
Tex2D.target = gl.GL_TEXTURE_2D
function Tex2D:create(args)
	gl.glTexImage2D(
		args.target or self.target,
		args.level or 0,
		args.internalFormat or gl.GL_RGB,
		args.width,
		args.height,
		args.border or 0,
		args.format or gl.GL_RGBA,
		args.type or gl.GL_FLOAT,
		args.data)
	-- store for later use
	self.width = args.width
	self.height = args.height
end
function Tex2D.load(args)	--alternative to Tex2D.new for file loading
	local self = setmetatable({}, Tex2D)
	if type(args) == 'string' then
		self.id = glush.createTex2D(args)	-- engine-bound function
	elseif type(args) == 'table' then
		self.id = glush.createTex2D{
			file=args.filename;
			unit=args.unit;
		}
		self:setArgs(args)
	else
		error('unknown filename param type')
	end
	self.width = gl.glGetTexLevelParameter(self.target, 0, gl.GL_TEXTURE_WIDTH)
	self.height = gl.glGetTexLevelParameter(self.target, 0, gl.GL_TEXTURE_HEIGHT)
	return self
end


TexCube = class(Tex)
TexCube.target = gl.GL_TEXTURE_CUBE_MAP
function TexCube:create(args)
	-- now args.data should be a 6-indexed array of whatever data you were gonna pass the texture
	local data = args.data or {}
	for i=1,6 do
		-- hijacking from Tex2D...
		Tex2D.create(self, table.new(args, {
			target=(args.target or gl.GL_TEXTURE_CUBE_MAP_POSITIVE_X) + i-1;
			data=data[i];
		}))
	end
end


Tex3D = class(Tex)
Tex3D.target = gl.GL_TEXTURE_3D
function Tex3D:create(args)
	gl.glTexImage3D(
		self.target,
		args.level or 0,
		args.internalFormat or gl.GL_RGB,
		args.width,
		args.height,
		args.depth,
		args.border or 0,
		args.format or gl.GL_RGBA,
		args.type or gl.GL_FLOAT,
		args.data)
	-- store for later use
	self.width = args.width
	self.height = args.height
	self.depth = args.depth
end

